// @(#) ENIB/LI2 (c) - C++ Utilities
//--------------------------------------------------------------------------//
// file        : blockAlc.h                                                 //
// description : header file for the ENIB/LI2 class BlockAllocator          //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#ifndef BLOCKALC_H
#define BLOCKALC_H

#ifndef __MSDOS__
// #include <stl_config.h>
#else
#pragma warning( disable : 4291 )
#endif

using namespace std;

#include <stdlib.h>

//--------------------------------------------------------------------------//
// class BlockAllocator                                                     //
//--------------------------------------------------------------------------//

class BlockAllocator
{
public:
// Static methods for memory management
  static void * allocate(size_t size);
  static void free(void * block);
  static void * realloc(void * block,size_t newSize);
  inline static void deallocate(void * block,size_t /* dummySize */)
    {
    BlockAllocator::free(block);
    }
  inline static void * reallocate(void * block,
                                  size_t /* dummyOldSize */,size_t newSize)
    {
    return(BlockAllocator::realloc(block,newSize));
    }
  static void lock(void);
  static void unlock(void);
// Allocators
  BlockAllocator(void);
  virtual ~BlockAllocator(void);
protected:                                 //---- use only static methods ----
// Private utilities
  inline static size_t _maxBlockSize(void);
  inline static size_t _blockIndex(size_t size);
  inline static size_t _blockSize(size_t size);
// Attributes
  static int _nbCreated;
  static const size_t _sizeOffset;
  static void * _blockList[];
  static BlockAllocator _allocator;
};

//--------------------------------------------------------------------------//
// class TypedAllocator                                                     //
//--------------------------------------------------------------------------//

template <class T>
class TypedAllocator : public BlockAllocator
{
public:
#ifdef __STL_MEMBER_TEMPLATES
  template <class T1> struct rebind { typedef TypedAllocator<T1> other; };
#endif
// Static methods for memory management
  inline static T * allocate(size_t size)
    {
    return((T *)BlockAllocator::allocate(size*sizeof(T)));
    }
  inline static void free(T * block)
    {
    BlockAllocator::free(block);
    }
  inline static T * realloc(T * block,size_t newSize)
    {
    return((T *)BlockAllocator::realloc(block,newSize*sizeof(T)));
    }
  inline static void deallocate(T * block,size_t /* dummySize */)
    {
    BlockAllocator::free(block);
    }
  inline static T * reallocate(T * block,
                               size_t /* dummyOldSize */,size_t newSize)
    {
    return((T *)BlockAllocator::realloc(block,newSize*sizeof(T)));
    }
// Allocators
  inline TypedAllocator(void) : BlockAllocator() {}
#ifdef __STL_MEMBER_TEMPLATES
  template <class T2> inline TypedAllocator(const T2 &) : BlockAllocator() {}
#endif
  inline virtual ~TypedAllocator(void) {}
};

//--------------------------------------------------------------------------//
// class BlockAllocatedObject                                               //
//--------------------------------------------------------------------------//

class BlockAllocatedObject
{
public:
  inline BlockAllocatedObject(void) {}
  inline virtual ~BlockAllocatedObject(void) {}
  inline void * operator new(size_t size)
         {
         return(BlockAllocator::allocate(size));
         }
  inline void * operator new(size_t,void * block)
         {
         return(block);
         }
#if defined (__MSDOS__) && defined(_DEBUG)
  inline void * operator new(size_t size,const char* , int)
         {
          return(BlockAllocator::allocate(size));
         }
#endif
  inline void operator delete(void * block,size_t size)
         {
         BlockAllocator::deallocate(block,size);
         }
};

#endif // BLOCKALC_H

