// @(#) ENIB/LI2 (c) - C++ Utilities
//--------------------------------------------------------------------------//
// file        : smplThrd.h                                                 //
// description : header file for the ENIB/LI2 class SimpleThread            //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#ifndef SMPLTHRD_H
#define SMPLTHRD_H

#include "Utility/stlMap.h"

#ifdef PTHREADS
#include <pthread.h>
#else
#ifdef __GNUG__
#include "Utility/ipcMutex.h"
#endif
#ifdef __sgi
#include <sys/types.h>
#include <sys/prctl.h>
#include <ulocks.h>
#endif
#ifdef __MSDOS__
typedef unsigned int UINT;
typedef void * LPVOID;
class CMutex;
class CWinThread;
#endif
#endif

//--------------------------------------------------------------------------//
// class SimpleThread                                                       //
//--------------------------------------------------------------------------//

class SimpleThread : public BlockAllocatedObject
{
public:
// Static Methods
  static SimpleThread * getCurrent(void);
  static SimpleThread * getThread(unsigned long id);
  static SimpleThread * getMain(void);
  static void yield(void);
  static void relaxThreads(void);
// Allocators
  SimpleThread(void * (*task)(void *),void * taskArg=(void *)0);
  virtual ~SimpleThread(void);
// Inspectors
  bool fail(void) const;
  unsigned long getID() const;
  bool isStarted(void) const;
  bool isEnded(void) const;
  bool isSuspended(void) const;
  void * getResult(void) const;
// Modifiers
  void start(void);
  void suspend(void);
  void resume(void);
protected:
// Private Methods
  SimpleThread(void);
#ifdef PTHREADS
  static void * _threadTask(void * data);
#else
#ifdef __GNUG__
  static int _threadTask(void * data);
#endif
#ifdef __sgi
  static void _threadTask(void * data);
#endif
#ifdef __MSDOS__
  static UINT _threadTask(LPVOID pParam) ;
#endif
#endif
// Attributes
  void * (*_task)(void *);
  void * _taskArg;
  unsigned long _id;
  bool _fail;
  bool _started;
  bool _ended;
  bool _suspended;
  void * _result;
#ifndef PTHREADS
#ifdef __GNUG__
  const size_t _stackSize;
  char * _stack;
#endif
#ifdef __MSDOS__
  CWinThread * _cwinThread;
#endif
#endif
  typedef StlMap<unsigned long,SimpleThread *> ThreadMap;
  static ThreadMap * _threadMap;
  static SimpleThread _mainThread;
};

//--------------------------------------------------------------------------//
// class SimpleThreadMutex                                                  //
//--------------------------------------------------------------------------//

class SimpleThreadMutex : public BlockAllocatedObject
{
public:
// Allocators
  SimpleThreadMutex(void);
  virtual ~SimpleThreadMutex(void);
// Inspectors
  bool fail(void) const;
// Modifiers
  void lock(void);
  bool tryLock(void);
  void unlock(void);
protected:
  SimpleThreadMutex(BlockAllocator * dummy);
// Attributes
#ifdef PTHREADS
  pthread_mutex_t _id;
  bool _fail;
#else
#ifdef __GNUG__
  IPCMutex _mutex;
#endif
#ifdef __sgi
  usema_t * _mutex;
  static usptr_t * _arena;
#endif
#ifdef __MSDOS__
  CMutex * _mutex;
#endif
#endif
};

#endif // SMPLTHRD_H

