// @(#) ENIB/LI2 (c) - C++ Utilities
//--------------------------------------------------------------------------//
// file        : stlList.h                                                  //
// description : header file for the ENIB/LI2 class StlList                 //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#ifndef STLLIST_H
#define STLLIST_H

#include "Utility/stlUtil.h"

#ifndef __MSDOS__
#include <list>
#else
#include <list>
#endif

//--------------------------------------------------------------------------//
// class StlList                                                            //
//--------------------------------------------------------------------------//

#ifndef __MSDOS__
template <class T>
class StlList : public BlockAllocatedObject,
                public list<T>
{
protected:
  typedef list<T> stl_type;
#else // Microsoft STL is too different from original STL to be fully used
template <class T>
class StlList : public BlockAllocatedObject,
                public std::list<T>
{
protected:
  typedef std::list<T> stl_type;
#endif // __MSDOS__
public:
  inline StlList(void)
   : BlockAllocatedObject(), stl_type() {}
  inline StlList(typename stl_type::size_type n, const T & value)
   : BlockAllocatedObject(), stl_type(n,value) {}
  inline StlList(typename stl_type::size_type n)
   : BlockAllocatedObject(), stl_type(n) {}
// #ifdef __STL_MEMBER_TEMPLATES
#if 1
  template <class InputIterator>
  inline StlList(InputIterator first,InputIterator last)
   : BlockAllocatedObject(), stl_type(first,last) {}
#else
  inline StlList(const T * first,const T * last)
   : BlockAllocatedObject(), stl_type(first,last) {}
  inline StlList(const_iterator first,const_iterator last)
   : BlockAllocatedObject(), stl_type(first,last) {}
# endif // __STL_MEMBER_TEMPLATES
  inline StlList(const stl_type & l)
   : BlockAllocatedObject(), stl_type(l) {}
  inline virtual ~StlList(void) {}
  inline StlList<T> & operator=(const stl_type & l)
    {
    stl_type::operator=(l);
    return(*this);
    }
};

#endif // STLLIST_H

