// @(#) ENIB/LI2 (c) - C++ Utilities
//--------------------------------------------------------------------------//
// file        : stlMap.h                                                   //
// description : header file for the ENIB/LI2 class StlMap                  //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#ifndef STLMAP_H
#define STLMAP_H

#include "Utility/stlUtil.h"

#ifndef __MSDOS__
#include <map>
#else
#include <map>
#endif

//--------------------------------------------------------------------------//
// class StlMap                                                             //
//--------------------------------------------------------------------------//

#ifndef __MSDOS__
template <class Key,class T,class Comp=StlLess<Key> >
class StlMap : public BlockAllocatedObject,
               public map<Key,T,Comp>
{
protected:
  typedef map<Key,T,Comp> stl_type;
#else // Microsoft STL is too different from original STL to be fully used
template <class Key,class T,class Comp=StlLess<Key> >
class StlMap : public BlockAllocatedObject,
               public std::map<Key,T,Comp>
{
protected:
  typedef std::map<Key,T,Comp> stl_type;
#endif // __MSDOS__
public:
  inline StlMap(void)
   : BlockAllocatedObject(), stl_type() {}
  inline StlMap(const Comp & comp)
   : BlockAllocatedObject(), stl_type(comp) {}
// #ifdef __STL_MEMBER_TEMPLATES
#if 1
  template <class InputIterator>
  inline StlMap(InputIterator first,InputIterator last)
   : BlockAllocatedObject(), stl_type(first,last) {}
  template <class InputIterator>
  inline StlMap(InputIterator first,InputIterator last,const Comp & comp)
   : BlockAllocatedObject(), stl_type(first,last,comp) {}
#else
  inline StlMap(const stl_type::value_type * first,
                const stl_type::value_type * last)
   : BlockAllocatedObject(), stl_type(first,last) {}
  inline StlMap(const stl_type::value_type * first,
                const stl_type::value_type * last,
                const Comp & comp)
   : BlockAllocatedObject(), stl_type(first,last,comp) {}
  inline StlMap(const_iterator first,const_iterator last)
   : BlockAllocatedObject(), stl_type(first,last) {}
  inline StlMap(const_iterator first,const_iterator last,const Comp & comp)
   : BlockAllocatedObject(), stl_type(first,last,comp) {}
#endif // __STL_MEMBER_TEMPLATES
  inline StlMap(const stl_type & m)
   : BlockAllocatedObject(), stl_type(m) {}
  inline virtual ~StlMap(void) {}
  inline StlMap<Key,T,Comp> & operator=(const stl_type & m)
    {
    stl_type::operator=(m);
    return(*this);
    }
};

#endif // STLMAP_H

