// @(#) ENIB/LI2 (c) - C++ Utilities
//--------------------------------------------------------------------------//
// file        : stlMuSet.h                                                 //
// description : header file for the ENIB/LI2 class StlMultiSet             //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#ifndef STLMUSET_H
#define STLMUSET_H

#include "Utility/stlUtil.h"

#ifndef __MSDOS__
#include <multiset>
#else
#include <set>
#endif

//--------------------------------------------------------------------------//
// class StlMultiSet                                                        //
//--------------------------------------------------------------------------//

#ifndef __MSDOS__
template <class Key,class Comp=StlLess<Key> >
class StlMultiSet : public BlockAllocatedObject,
                    public multiset<Key,Comp>
{
protected:
  typedef multiset<Key,Comp> stl_type;
#else // Microsoft STL is too different from original STL to be fully used
template <class Key,class Comp=StlLess<Key> >
class StlMultiSet : public BlockAllocatedObject,
                    public std::multiset<Key,Comp>
{
protected:
  typedef std::multiset<Key,Comp> stl_type;
#endif // __MSDOS__
public:
  inline StlMultiSet(void)
   : BlockAllocatedObject(), stl_type() {}
  inline StlMultiSet(const Comp & comp)
   : BlockAllocatedObject(), stl_type(comp) {}
#ifdef __STL_MEMBER_TEMPLATES
  template <class InputIterator>
  inline StlMultiSet(InputIterator first,InputIterator last)
   : BlockAllocatedObject(), stl_type(first,last) {}
  template <class InputIterator>
  inline StlMultiSet(InputIterator first,InputIterator last,const Comp & comp)
   : BlockAllocatedObject(), stl_type(first,last,comp) {}
#else
  inline StlMultiSet(const stl_type::value_type * first,
                     const stl_type::value_type * last)
   : BlockAllocatedObject(), stl_type(first,last) {}
  inline StlMultiSet(const stl_type::value_type * first,
                     const stl_type::value_type * last,
                     const Comp & comp)
   : BlockAllocatedObject(), stl_type(first,last,comp) {}
  inline StlMultiSet(const_iterator first,const_iterator last)
   : BlockAllocatedObject(), stl_type(first,last) {}
  inline StlMultiSet(const_iterator first,const_iterator last,
                     const Comp & comp)
   : BlockAllocatedObject(), stl_type(first,last,comp) {}
#endif // __STL_MEMBER_TEMPLATES
  inline StlMultiSet(const stl_type & s)
   : BlockAllocatedObject(), stl_type(s) {}
  inline virtual ~StlMultiSet(void) {}
  inline StlMultiSet<Key,Comp> & operator=(const stl_type & s)
    {
    stl_type::operator=(s);
    return(*this);
    }
};

#endif // STLMUSET_H

