// @(#) ENIB/LI2 (c) - C++ Utilities
//--------------------------------------------------------------------------//
// file        : timer.h                                                    //
// description : header file for the ENIB/LI2 class Timer                   //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#ifndef TIMER_H
#define TIMER_H

#ifndef __MSDOS__
#include <unistd.h>
#include <sys/time.h>
#else
#include <sys/timeb.h>
#include <windows.h>
#include <winbase.h>
#endif

#include "Utility/smplVect.h"
#include "Utility/stlList.h"

//--------------------------------------------------------------------------//
// class Timer                                                              //
//--------------------------------------------------------------------------//

class Timer : public BlockAllocatedObject
{
public:
  typedef void (*Callback)(void *);
// Allocators
  Timer(void);
  virtual ~Timer(void);
// Time management
  inline static void sleep(unsigned long ms);
  inline static unsigned long getTime(void);
  unsigned long addTimeOut(unsigned long ms,Timer::Callback timerCB,
                           void * timerData=(void *)0);
  bool removeTimeOut(unsigned long id);
protected:
  static void _timerHandler(int sigNum);
  static void _stop(void);
  static void _relaunch(unsigned long date);
// Attributes
  static int _nbTimers;
  static unsigned long _currentId;
#ifndef __MSDOS__
  static struct sigaction _newAction;
  static struct sigaction _oldAction;
#endif
  class Job : public BlockAllocatedObject
    {
    public:
      unsigned long id;
      unsigned long date;
      Callback cb;
      void * data;
    };
  static SimpleVector<Job *> _jobStock;
  static StlList<Job *> _jobs;
};

#include "Utility/timer.Ci"

#endif // TIMER_H

