// @(#) ENIB/LI2 (c) - C++ Utilities
//--------------------------------------------------------------------------//
// file        : ipcMutex.cpp                                               //
// description : source file for the ENIB/LI2 class IPCMutex                //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//


#include "Utility/ipcMutex.h"

#include <iostream>

#ifndef __MSDOS__
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/sem.h>
#include <errno.h>
#ifdef _SEM_SEMUN_UNDEFINED
union semun
{
int val;
struct semid_ds * buf;
unsigned short int *array;
struct seminfo *__buf;
};
#endif
#endif

//--------------------------------------------------------------------------//
// class IPCMutex                                                           //
//--------------------------------------------------------------------------//

SimpleVector<IPCMutex *> IPCMutex::_mtxs;

IPCMutex * IPCMutex::_allocMutex=(IPCMutex *)0;

//-------- Static Methods --------------------------------------------------//
void IPCMutex::relaxIPCs(void) // static
{
#ifndef __MSDOS__
union semun the_semun;
the_semun.val=0;
for(int i=_mtxs.getSize();i--;)
  {
  if(_allocMutex)
    {
    if(_allocMutex->_id>=0)
      ::semctl(_allocMutex->_id,0,IPC_RMID,the_semun);
    _allocMutex=(IPCMutex *)0;
    }
  if(_mtxs[i])
    {
    if(_mtxs[i]->_id>=0)
      ::semctl(_mtxs[i]->_id,0,IPC_RMID,the_semun);
    _mtxs[i]=(IPCMutex *)0;
    }
  }
#endif
}

//-------- Allocators ------------------------------------------------------//

IPCMutex::IPCMutex(void)
 : BlockAllocatedObject(),
   _id(-1), _creator(true)
{
#ifndef __MSDOS__
_id=::semget(IPC_PRIVATE,1,0666);
if(_id>=0)
  {
  union semun the_semun;
  the_semun.val=1;
  ::semctl(_id,0,SETVAL,the_semun);
  _mtxs.pushBack(this);
  }
#endif
}

IPCMutex::IPCMutex(BlockAllocator * /* dummy */) // protected
 : BlockAllocatedObject(),
   _id(-1), _creator(true)
{
#ifndef __MSDOS__
_id=::semget(IPC_PRIVATE,1,0666);
if(_id>=0)
  {
  union semun the_semun;
  the_semun.val=1;
  ::semctl(_id,0,SETVAL,the_semun);
  _allocMutex=this;
  }
#endif
}

IPCMutex::IPCMutex(int id)
 : BlockAllocatedObject(),
   _id(id), _creator(false)
{}

IPCMutex::~IPCMutex(void) // virtual
{
if(_creator&&(_id>=0))
  {
#ifndef __MSDOS__
  union semun the_semun;
  the_semun.val=0;
  ::semctl(_id,0,IPC_RMID,the_semun);
  if(_allocMutex==this) _allocMutex=(IPCMutex *)0;
  else
    {
    for(int i=_mtxs.getSize();i--;)
      {
      if(_mtxs[i]==this)
        {
        _mtxs.erase(i);
        break;
        }
      }
    }
#endif
  }
}

//-------- Inspectors ------------------------------------------------------//
bool IPCMutex::fail(void) const
{
return(_id<0);
}

int IPCMutex::getID(void) const
{
return(_id);
}

bool IPCMutex::isLocked(void) const
{
if(_id>=0)
  {
#ifndef __MSDOS__
  union semun the_semun;
  the_semun.val=0;
  return(!::semctl(_id,0,GETVAL,the_semun));
#endif
  }
return(true);
}

//-------- Modifiers -------------------------------------------------------//
void IPCMutex::lock(void)
{
if(_id>=0)
  {
#ifndef __MSDOS__
  struct sembuf the_sembuf;
  the_sembuf.sem_num=0;
  the_sembuf.sem_op=-1;
  // the_sembuf.sem_flg=SEM_UNDO;
  the_sembuf.sem_flg=0;
  while((::semop(_id,&the_sembuf,1)<0)&&(errno==EINTR));
#endif
  }
}

bool IPCMutex::tryLock(void)
{
if(_id>=0)
  {
#ifndef __MSDOS__
  struct sembuf the_sembuf;
  the_sembuf.sem_num=0;
  the_sembuf.sem_op=-1;
  the_sembuf.sem_flg=IPC_NOWAIT;
  int r;
  do
    {
    r=::semop(_id,&the_sembuf,1);
    } while((r<0)&&(errno==EINTR));
  return(r>=0);
#endif
  }
return(false);
}

void IPCMutex::unlock(void)
{
if(_id>=0)
  {
#ifndef __MSDOS__
  struct sembuf the_sembuf;
  the_sembuf.sem_num=0;
  the_sembuf.sem_op=1;
  // the_sembuf.sem_flg=SEM_UNDO;
  the_sembuf.sem_flg=0;
  while((::semop(_id,&the_sembuf,1)<0)&&(errno==EINTR));
#endif
  }
}

