// @(#) ENIB/LI2 (c) - C++ Utilities
//--------------------------------------------------------------------------//
// file        : ipcShMem.cpp                                               //
// description : source file for the ENIB/LI2 class IPCSharedMemory         //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//


#include "Utility/ipcShMem.h"

#ifndef __MSDOS__
#include <unistd.h>
#endif

//--------------------------------------------------------------------------//
// class IPCSharedMemory                                                    //
//--------------------------------------------------------------------------//

SimpleVector<IPCSharedMemory *> IPCSharedMemory::_mems;

//-------- Static Methods --------------------------------------------------//
void IPCSharedMemory::relaxIPCs(void) // static
{
#ifndef __MSDOS__
for(int i=_mems.getSize();i--;)
  {
  if(_mems[i])
    {
    if(_mems[i]->_address)
      ::shmdt((char *)_mems[i]->_address);
    if(_mems[i]->_id>=0)
      ::shmctl(_mems[i]->_id,IPC_RMID,&_mems[i]->_shmid_ds);
    _mems[i]=(IPCSharedMemory *)0;
    }
  }
#endif
}

//-------- Allocators ------------------------------------------------------//

IPCSharedMemory::IPCSharedMemory(size_t size)
 : BlockAllocatedObject(),
   _id(-1), _creator(true), _size(size), _address((void *)0)
{
#ifndef __MSDOS__
_id=::shmget(IPC_PRIVATE,_size,0666);
if((_id>=0)&&!::shmctl(_id,IPC_STAT,&_shmid_ds))
  {
  _creator=((_shmid_ds.shm_cpid==getpid())&&!_shmid_ds.shm_nattch);
  _size=_shmid_ds.shm_segsz;
  _address=(void *)::shmat(_id,(char *)0,SHM_RND);
  if(_creator) _mems.pushBack(this);
  }
#endif
}

IPCSharedMemory::IPCSharedMemory(int id)
 : BlockAllocatedObject(),
   _id(id), _creator(false), _size(0), _address((void *)0)
{
#ifndef __MSDOS__
if((_id>=0)&&!::shmctl(_id,IPC_STAT,&_shmid_ds))
  {
  _creator=((_shmid_ds.shm_cpid==getpid())&&!_shmid_ds.shm_nattch);
  _size=_shmid_ds.shm_segsz;
  _address=(void *)::shmat(_id,(char *)0,SHM_RND);
  if(_creator) _mems.pushBack(this);
  }
#endif
}

IPCSharedMemory::~IPCSharedMemory(void) // virtual
{
#ifndef __MSDOS__
if(_address) ::shmdt((char *)_address);
if((_id>=0)&&_creator)
  {
  ::shmctl(_id,IPC_RMID,&_shmid_ds);
  for(int i=_mems.getSize();i--;)
    {
    if(_mems[i]==this)
      {
      _mems.erase(i);
      break;
      }
    }
  }
#endif
}

//-------- Inspectors ------------------------------------------------------//
bool IPCSharedMemory::fail(void) const
{
return(!_address);
}

int IPCSharedMemory::getID(void) const
{
return(_id);
} 

size_t IPCSharedMemory::getSize(void) const
{
return(_size);
}

void * IPCSharedMemory::getAddress(void) const
{
return(_address);
}

//-------- Modifiers -------------------------------------------------------//

