// @(#) ENIB/LI2 (c) - C++ Utilities
//--------------------------------------------------------------------------//
// file        : smplThrd.cpp                                               //
// description : source file for the ENIB/LI2 class SimpleThread            //
// author(s)   : F. Harrouet                                                //
//             : P. Reignier (partie windows)                               //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//


#ifndef __MSDOS__
extern "C" {
#include <sched.h>
}
#include <signal.h>
#include <unistd.h>
#include <sys/wait.h>
#else
#pragma warning (disable : 4786 4800 ) 
#include "StdAfx.h"
#include <afxmt.h>
#endif

#include "Utility/smplThrd.h"

//--------------------------------------------------------------------------//
// class SimpleThread                                                       //
//--------------------------------------------------------------------------//

SimpleThread::ThreadMap * SimpleThread::_threadMap;

SimpleThread SimpleThread::_mainThread;

//-------- Static Methods --------------------------------------------------//
SimpleThread * SimpleThread::getCurrent(void) // static
{
#ifdef PTHREADS
return(SimpleThread::getThread((unsigned long)::pthread_self()));
#else
#ifndef __MSDOS__
return(SimpleThread::getThread((unsigned long)::getpid()));
#else
return(SimpleThread::getThread((unsigned long)AfxGetThread()->m_nThreadID));
#endif
#endif
}

SimpleThread * SimpleThread::getThread(unsigned long id) // static
{
ThreadMap::iterator it=SimpleThread::_threadMap->find(id);
if(it==SimpleThread::_threadMap->end()) return((SimpleThread *)0);
else return((*it).second);
}

SimpleThread * SimpleThread::getMain(void) // static
{
return(&SimpleThread::_mainThread);
}

void SimpleThread::yield(void) // static
{
#ifndef __MSDOS__
::sched_yield();
#else
::Sleep(0);
#endif
}

void SimpleThread::relaxThreads(void) // static
{
SimpleThread * current=SimpleThread::getCurrent();
SimpleThread::ThreadMap::iterator it=SimpleThread::_threadMap->begin();
for(;it!=SimpleThread::_threadMap->end();it++)
  {
  if((*it).second&&((*it).second!=current))
    {
#ifdef PTHREADS
    while(::pthread_cancel((pthread_t)(*it).second->_id))
      {
      SimpleThread::yield();
      }
#else
#ifndef __MSDOS__
    while(::kill((pid_t)(*it).second->_id,SIGKILL))
      {
      SimpleThread::yield();
      }
#else
    (*it).second->_cwinThread->ExitInstance();
#endif
#endif
    (*it).second=(SimpleThread *)0;
    }
  }
}

//-------- Allocators ------------------------------------------------------//
SimpleThread::SimpleThread(void * (*task)(void *),void * taskArg)
 : BlockAllocatedObject(),
   _task(task), _taskArg(taskArg), _id(0),
   _fail(false), _started(false), _ended(false),
   _suspended(false), _result((void *)0)
#ifndef PTHREADS
#ifdef __GNUG__
  ,_stackSize(32768), _stack((char *)0)
#endif
#endif
{
#ifdef PTHREADS
pthread_t thread_id;
if(::pthread_create(&thread_id,(pthread_attr_t *)0,
                    SimpleThread::_threadTask,(void *)this))
  _fail=true;
else _id=(unsigned long)thread_id;
#else // PTHREADS
#ifdef __GNUG__
_stack=(char *)BlockAllocator::allocate(_stackSize);
int thread_id=::clone(SimpleThread::_threadTask,(void *)(_stack+_stackSize),
                      CLONE_VM | CLONE_FS | CLONE_FILES |
                      CLONE_SIGHAND | SIGCHLD,(void *)this);
SimpleThread::yield();
if(thread_id<0) _fail=true;
else _id=(unsigned long)thread_id;
#endif
#ifdef __sgi
pid_t thread_id=::sproc(SimpleThread::_threadTask,PR_SALL,(void *)this);
if(thread_id<0) _fail=true;
else _id=(unsigned long)thread_id;
#endif
#ifdef __MSDOS__
_cwinThread=::AfxBeginThread(SimpleThread::_threadTask,(void *)this);
if(_cwinThread)
  {
  _id=(unsigned long)_cwinThread->m_nThreadID;
  _cwinThread->m_bAutoDelete=true;
  }
else _fail=true;
#endif
#endif
if(!_fail)
  {
  SimpleThread::_threadMap->insert(ThreadMap::value_type(_id,this));
  while(!_suspended) SimpleThread::yield();
  }
}

SimpleThread::~SimpleThread(void)
{
if(!_fail)
  {
  if(_suspended)
    {
    _fail=true;
#ifdef PTHREADS
    ::pthread_kill((pthread_t)_id,SIGCONT);
#else
#ifndef __MSDOS__
    ::kill((pid_t)_id,SIGCONT);
#else
    if(!_ended) _cwinThread->ResumeThread();
#endif
#endif
    }
  SimpleThread::_threadMap->erase(_id);
  if(_id==SimpleThread::_mainThread._id)
    {
    while(SimpleThread::_threadMap->size())
      {
      delete (*(SimpleThread::_threadMap->begin())).second;
      }
    }
  else
    {
#ifdef PTHREADS
    while(!::pthread_cancel((pthread_t)_id))
      {
      ::pthread_kill((pthread_t)_id,SIGCONT);
      SimpleThread::yield();
      }
#else
#ifndef __MSDOS__
    while(!::kill((pid_t)_id,SIGKILL))
      {
      ::kill((pid_t)_id,SIGCONT);
      SimpleThread::yield();
      }
#ifdef __GNUG__
    BlockAllocator::free(_stack);
#endif
#else
    if(!_ended) _cwinThread->ExitInstance();
    SimpleThread::yield();
#endif
#endif
    }
  }
}

//-------- Inspectors ------------------------------------------------------//
bool SimpleThread::fail(void) const
{
return(_fail);
}

unsigned long SimpleThread::getID(void) const
{
return(_id);
}

bool SimpleThread::isStarted(void) const
{
return(_started);
}

bool SimpleThread::isEnded(void) const
{
return(_ended);
}

bool SimpleThread::isSuspended(void) const
{
return(_suspended);
}

void * SimpleThread::getResult(void) const
{
#ifdef PTHREADS
if(!_fail&&(::pthread_self()!=(pthread_t)_id)) // avoid waiting itself
#else
#ifndef __MSDOS__
if(!_fail&&(::getpid()!=(pid_t)_id)) // avoid waiting itself
#else
if(!_fail&&                           // avoid waiting itself
   ((unsigned long)AfxGetThread()->m_nThreadID!=(unsigned long)_id))
#endif
#endif
  {
  while(!_ended)            // caller waits termination
    SimpleThread::yield();  // not pthread_join --> many waiting threads
  }
return(_result);
}

//-------- Modifiers -------------------------------------------------------//
void SimpleThread::start(void)
{
if(!_started) resume();
}

void SimpleThread::suspend(void)
{
if((!_fail)&&(!_ended)&&(!_suspended))
  {
  _suspended=true;
#ifdef PTHREADS
  ::pthread_kill((pthread_t)_id,SIGSTOP);
#else
#ifndef __MSDOS__
  ::kill((pid_t)_id,SIGSTOP);
#else
  _cwinThread->SuspendThread();
#endif
#endif
  }
}

void SimpleThread::resume(void)
{
if((!_fail)&&(!_ended)&&_suspended)
  {
  _suspended=false;
#ifdef PTHREADS
  ::pthread_kill((pthread_t)_id,SIGCONT);
#else
#ifndef __MSDOS__
  ::kill((pid_t)_id,SIGCONT);
#else
  _cwinThread->ResumeThread();
#endif
#endif
  }
}

//-------- Private Methods -------------------------------------------------//
SimpleThread::SimpleThread(void)
 : BlockAllocatedObject(),
   _task((void * (*)(void *))0), _taskArg((void *)0),
   _id(0), _fail(false), _started(true), _ended(false),
   _suspended(false), _result((void *)0)
{
#ifdef PTHREADS
_id=(unsigned long)::pthread_self();
::pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS,(int *)0);
::pthread_setcancelstate(PTHREAD_CANCEL_ENABLE,(int *)0);
#else
#ifndef __MSDOS__
_id=(unsigned long)::getpid();
#else
_cwinThread=(CWinThread*)0;
_id=AfxGetThread()->m_nThreadID ;
#endif
#endif
SimpleThread::_threadMap=new ThreadMap;
SimpleThread::_threadMap->insert(ThreadMap::value_type(_id,this));
}

#ifdef PTHREADS
void * SimpleThread::_threadTask(void * data) // static
#else
#ifdef __GNUG__
int SimpleThread::_threadTask(void * data) // static
#endif
#ifdef __sgi
void SimpleThread::_threadTask(void * data) // static
#endif
#ifdef __MSDOS__
UINT SimpleThread::_threadTask(LPVOID data) // static
#endif
#endif
{
SimpleThread * thread=(SimpleThread *)data;
#ifdef PTHREADS
thread->_id=(unsigned long)::pthread_self();
::pthread_detach((pthread_t)thread->_id);
::pthread_setcancelstate(PTHREAD_CANCEL_ENABLE,(int *)0);
::pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS,(int *)0);
#else
#ifndef __MSDOS__
thread->_id=(unsigned long)::getpid();
#else
thread->_id=AfxGetThread()->m_nThreadID;
#endif
#endif
thread->suspend();
thread->_started=true;
thread->_result=(*(thread->_task))(thread->_taskArg);
thread->_ended=true;
#ifdef PTHREADS
return((void *)0);
#else
#ifdef __GNUG__
::kill((pid_t)thread->_id,SIGKILL);
do
  {
  SimpleThread::yield();
  } while(!::kill((pid_t)thread->_id,0));
return(0);
#endif
#ifdef __MSDOS__
return(0);
#endif
#endif
}

//--------------------------------------------------------------------------//
// class SimpleThreadMutex                                                  //
//--------------------------------------------------------------------------//

#ifndef PTHREADS
#ifdef __sgi
usptr_t * SimpleThreadMutex::_arena=(usptr_t *)0;
#endif
#endif

//-------- Allocators ------------------------------------------------------//
SimpleThreadMutex::SimpleThreadMutex(void)
 : BlockAllocatedObject(),
#ifdef PTHREADS
   _id(), _fail(false)
{
if(::pthread_mutex_init(&_id,(pthread_mutexattr_t *)0)) _fail=true;
}
#else
#ifdef __GNUG__
   _mutex()
{
}
#endif
#ifdef __sgi
   _mutex((usema_t *)0)
{
if(!SimpleThreadMutex::_arena)
  {
  ::usconfig(CONF_INITUSERS,(unsigned int)1000);
  SimpleThreadMutex::_arena=::usinit("/dev/zero");
  }
if(SimpleThreadMutex::_arena)
  _mutex=::usnewsema(SimpleThreadMutex::_arena,1);
}
#endif
#ifdef __MSDOS__
   _mutex(new CMutex) 
{
}
#endif
#endif

SimpleThreadMutex::SimpleThreadMutex(BlockAllocator * /* dummy */ )
 : BlockAllocatedObject(),                                        // protected
#ifdef PTHREADS
   _id(), _fail(false)
{
if(::pthread_mutex_init(&_id,(pthread_mutexattr_t *)0)) _fail=true;
}
#else
#ifdef __GNUG__
   _mutex((BlockAllocator *)0)
{
}
#endif
#ifdef __sgi
   _mutex((usema_t *)0)
{
if(!SimpleThreadMutex::_arena)
  {
  ::usconfig(CONF_INITUSERS,(unsigned int)1000);
  SimpleThreadMutex::_arena=::usinit("/dev/zero");
  }
if(SimpleThreadMutex::_arena)
  _mutex=::usnewsema(SimpleThreadMutex::_arena,1);
}
#endif
#ifdef __MSDOS__
   _mutex(new CMutex)
{
}
#endif
#endif

SimpleThreadMutex::~SimpleThreadMutex(void) // virtual
{
#ifdef PTHREADS
if(!_fail) ::pthread_mutex_destroy(&_id);
#else
#ifdef __sgi
if(_mutex) ::usfreesema(_mutex,SimpleThreadMutex::_arena);
#endif
#ifdef __MSDOS__
delete _mutex;
#endif
#endif
}

//-------- Inspectors ------------------------------------------------------//
bool SimpleThreadMutex::fail(void) const
{
#ifdef PTHREADS
return(_fail);
#else
#ifdef __GNUG__
return(_mutex.fail());
#endif
#ifdef __sgi
return(_mutex ? false : true);
#endif
#ifdef __MSDOS__
return(false);
#endif
#endif
}

//-------- Modifiers -------------------------------------------------------//
void SimpleThreadMutex::lock(void)
{
#ifdef PTHREADS
::pthread_mutex_lock(&_id);
#else
#ifdef __GNUG__
_mutex.lock();
#endif
#ifdef __sgi
if(_mutex) ::uspsema(_mutex);
#endif
#ifdef __MSDOS__
_mutex->Lock();
#endif
#endif
}

bool SimpleThreadMutex::tryLock(void)
{
#ifdef PTHREADS
return(::pthread_mutex_trylock(&_id));
#else
#ifdef __GNUG__
return(_mutex.tryLock());
#endif
#ifdef __sgi
if(_mutex) return((bool)::uscpsema(_mutex));
else return(false);
#endif
#ifdef __MSDOS__
return((bool)_mutex->Lock(0));
#endif
#endif
}

void SimpleThreadMutex::unlock(void)
{
#ifdef PTHREADS
::pthread_mutex_unlock(&_id);
#else
#ifdef __GNUG__
_mutex.unlock();
#endif
#ifdef __sgi
if(_mutex) ::usvsema(_mutex);
#endif
#ifdef __MSDOS__
_mutex->Unlock();
#endif
#endif
}

