// @(#) ENIB/LI2 (c) - C++ Utilities
//--------------------------------------------------------------------------//
// file        : system.cpp                                                 //
// description : source file for the ENIB/LI2 class System                  //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "Utility/system.h"

#ifndef __MSDOS__
#include <dlfcn.h>
#include <unistd.h>
#include <sys/wait.h>
#include <signal.h>
#include <errno.h>
#else
#include <windows.h>
#endif

//--------------------------------------------------------------------------//
// class System                                                             //
//--------------------------------------------------------------------------//

System::System(void) {} // protected
System::~System(void) {} // protected

//-------- Exit Handler ----------------------------------------------------//
bool (* System::_userHandler)(int)=(bool (*)(int))0;

void System::_exitHandler(int signum) // static
{
#ifndef __MSDOS__
bool mustExit=true;
if(System::_userHandler) mustExit=(*System::_userHandler)(signum);
if(mustExit)
  {
  cerr << "Process " << getpid() << " exiting on signal " << signum << endl;
  ::kill(getpid(),SIGKILL);
  }
#endif
}

void System::installExitHandler(bool (*handler)(int)) // static
{
#ifndef __MSDOS__
System::_userHandler=handler;
SimpleVector<int> vect;
#ifdef __GNUG__
vect.pushBack(SIGIOT);
vect.pushBack(SIGSTKFLT);
vect.pushBack(SIGUNUSED);
#else
vect.pushBack(SIGSYS);
#endif
vect.pushBack(SIGHUP);
vect.pushBack(SIGINT);
vect.pushBack(SIGQUIT);
vect.pushBack(SIGILL);
vect.pushBack(SIGTRAP);
vect.pushBack(SIGABRT);
vect.pushBack(SIGFPE);
vect.pushBack(SIGBUS);
vect.pushBack(SIGSEGV);
vect.pushBack(SIGPIPE);
vect.pushBack(SIGALRM);
vect.pushBack(SIGTERM);
vect.pushBack(SIGUSR1);
vect.pushBack(SIGUSR2);
vect.pushBack(SIGPWR);
vect.pushBack(SIGPOLL);
vect.pushBack(SIGIO);
vect.pushBack(SIGVTALRM);
vect.pushBack(SIGPROF);
vect.pushBack(SIGXCPU);
vect.pushBack(SIGXFSZ);
struct sigaction act;
sigemptyset(&act.sa_mask);
act.sa_handler=System::_exitHandler;
act.sa_flags=0;
for(unsigned int i=0;i<vect.getSize();i++)
  {
  sigaction(vect[i],&act,(struct sigaction *)0);
  }
#endif
}

void System::ignoreTimers(void) // static
{
#ifndef __MSDOS__
struct sigaction act;
sigemptyset(&act.sa_mask);
act.sa_handler=SIG_IGN;
act.sa_flags=0;
sigaction(SIGALRM,&act,(struct sigaction *)0);
sigaction(SIGVTALRM,&act,(struct sigaction *)0);
sigaction(SIGPROF,&act,(struct sigaction *)0);
#endif
}


//-------- Dynamic Libraries -----------------------------------------------//
void * System::openDynLib(const Strings & libName) // static
{
const char * name=(const char *)0;
if(libName.len()) name=libName;
#ifndef __MSDOS__
void * handle=::dlopen(name,RTLD_LAZY|RTLD_GLOBAL);
if(!handle) cerr << dlerror() << endl;
return(handle);
#else
return(LoadLibrary(name));
#endif
}

void * System::getDynLibSymbol(void * handle,const Strings & symbName)
{                                                                  // static
#ifndef __MSDOS__
return(::dlsym(handle,symbName));
#else
return(GetProcAddress((HMODULE)handle,symbName));
#endif
}

int System::closeDynLib(void * handle) // static
{
#ifndef __MSDOS__
return(::dlclose(handle));
#else
return(!FreeLibrary((HMODULE)handle));
#endif
}

//-------- Command Launching -----------------------------------------------//
int System::launch(const Strings & command) // static
{
#ifndef __MSDOS__
return(::system(command));
#else
  STARTUPINFO startupInfo  ;
  GetStartupInfo(&startupInfo) ;
  PROCESS_INFORMATION info ;
  char *com = (char *) (const char *) command ;
  int success1=CreateProcess(NULL,com, NULL, NULL, false, 
                            DETACHED_PROCESS, NULL,NULL,&startupInfo,&info) ;
  int success2 = 1 ;
  if (!success1)
  // tries if it is a builtIn command
  {
    success2 = ::system(command) ;
  }
  else
  {
	   // wait for the other application to start
	  if (WaitForInputIdle(info.hProcess,INFINITE))
	  {
		  DWORD error = GetLastError() ;
		  if (error)
		  {
			  LPVOID lpMsgBuf;
			  FormatMessage( 
					FORMAT_MESSAGE_ALLOCATE_BUFFER | 
					FORMAT_MESSAGE_FROM_SYSTEM | 
					FORMAT_MESSAGE_IGNORE_INSERTS,
					NULL,
					error,
					MAKELANGID(LANG_NEUTRAL,
					 SUBLANG_DEFAULT), // Default language
					(LPTSTR) &lpMsgBuf,
					0,
					NULL 
				);
			  // Process any inserts in lpMsgBuf.
			  // ...
			  // Display the string.
			  MessageBox( NULL, (LPCTSTR)lpMsgBuf, "Error",
				      MB_OK | MB_ICONINFORMATION );
			  // Free the buffer.
			  LocalFree( lpMsgBuf );
		  }
		  else
		  // WaitForInputIdle may have failed because the
		  // application might be a console application
		  // we wait 6 seconds for it to be lauched
     		  ::Sleep(6000) ;
	  }
  }

  return (success1 || success2) ;
#endif // __MSDOS__
}

void System::launchI(const Strings & command,
                     const Strings & input) // static
{
#ifndef __MSDOS__
int input_p[2];
if(::pipe(input_p)==-1)
  {
  return;
  }
int r;
do
  {
  r=::write(input_p[1],(const char *)input,input.len());
  } while((r==-1)&&(errno==EINTR));
if(r==-1)
  {
  ::close(input_p[0]);
  ::close(input_p[1]);
  return;
  }
int pid=fork();
switch(pid)
  {
  case -1:
    {
    ::close(input_p[0]);
    ::close(input_p[1]);
    return;
    }
  case 0: // child
    {
    System::ignoreTimers();
    ::dup2(input_p[0],STDIN_FILENO);
    ::close(input_p[0]);
    ::close(input_p[1]);
    char * argv[4];
    argv[0]="/bin/sh";
    argv[1]="-c";
    argv[2]=(char *)(const char *)command;
    argv[3]=(char *)0;
    ::execv(argv[0],argv);
    ::exit(127); // should never be reached
    }
  default:
    {
    ::close(input_p[1]);
    ::close(input_p[0]);
    int status;
    ::waitpid(pid,&status,0);
    }
  }
#endif
}

Strings System::launchO(const Strings & command) // static
{
Strings result;
#ifndef __MSDOS__
int output_p[2];
if(::pipe(output_p)==-1)
  {
  return(result);
  }
int pid=fork();
switch(pid)
  {
  case -1:
    {
    ::close(output_p[0]);
    ::close(output_p[1]);
    return(result);
    }
  case 0: // child
    {
    System::ignoreTimers();
    ::dup2(output_p[1],STDOUT_FILENO);
    ::close(output_p[0]);
    ::close(output_p[1]);
    char * argv[4];
    argv[0]="/bin/sh";
    argv[1]="-c";
    argv[2]=(char *)(const char *)command;
    argv[3]=(char *)0;
    ::execv(argv[0],argv);
    ::exit(127); // should never be reached
    }
  default:
    {
    ::close(output_p[1]);
    int status;
    ::waitpid(pid,&status,0);
    int r;
    char c;
    do
      {
      do
        {
        r=::read(output_p[0],&c,sizeof(char));
        } while((r==-1)&&(errno==EINTR));
      if(r&&(r!=-1)) result+=c;
      } while(r==sizeof(char));
    ::close(output_p[0]);
    if(result.len()&&(result[result.len()-1]=='\n'))
      result=result.subString(0,result.len()-1);
    }
  }
#endif
return(result);
}

Strings System::launchE(const Strings & command) // static
{
Strings result;
#ifndef __MSDOS__
int error_p[2];
if(::pipe(error_p)==-1)
  {
  return(result);
  }
int pid=fork();
switch(pid)
  {
  case -1:
    {
    ::close(error_p[0]);
    ::close(error_p[1]);
    return(result);
    }
  case 0: // child
    {
    System::ignoreTimers();
    ::dup2(error_p[1],STDERR_FILENO);
    ::close(error_p[0]);
    ::close(error_p[1]);
    char * argv[4];
    argv[0]="/bin/sh";
    argv[1]="-c";
    argv[2]=(char *)(const char *)command;
    argv[3]=(char *)0;
    ::execv(argv[0],argv);
    ::exit(127); // should never be reached
    }
  default:
    {
    ::close(error_p[1]);
    int status;
    ::waitpid(pid,&status,0);
    int r;
    char c;
    do
      {
      do
        {
        r=::read(error_p[0],&c,sizeof(char));
        } while((r==-1)&&(errno==EINTR));
      if(r&&(r!=-1)) result+=c;
      } while(r==sizeof(char));
    ::close(error_p[0]);
    if(result.len()&&(result[result.len()-1]=='\n'))
      result=result.subString(0,result.len()-1);
    }
  }
#endif
return(result);
}

Strings System::launchIO(const Strings & command,
                         const Strings & input) // static
{
Strings result;
#ifndef __MSDOS__
int input_p[2];
int output_p[2];
if(::pipe(input_p)==-1)
  {
  return(result);
  }
if(::pipe(output_p)==-1)
  {
  ::close(input_p[0]);
  ::close(input_p[1]);
  return(result);
  }
int r;
do
  {
  r=::write(input_p[1],(const char *)input,input.len());
  } while((r==-1)&&(errno==EINTR));
if(r==-1)
  {
  ::close(input_p[0]);
  ::close(input_p[1]);
  ::close(output_p[0]);
  ::close(output_p[1]);
  return(result);
  }
int pid=fork();
switch(pid)
  {
  case -1:
    {
    ::close(input_p[0]);
    ::close(input_p[1]);
    ::close(output_p[0]);
    ::close(output_p[1]);
    return(result);
    }
  case 0: // child
    {
    System::ignoreTimers();
    ::dup2(output_p[1],STDOUT_FILENO);
    ::close(output_p[0]);
    ::close(output_p[1]);
    ::dup2(input_p[0],STDIN_FILENO);
    ::close(input_p[0]);
    ::close(input_p[1]);
    char * argv[4];
    argv[0]="/bin/sh";
    argv[1]="-c";
    argv[2]=(char *)(const char *)command;
    argv[3]=(char *)0;
    ::execv(argv[0],argv);
    ::exit(127); // should never be reached
    }
  default:
    {
    ::close(input_p[1]);
    ::close(output_p[1]);
    ::close(input_p[0]);
    int status;
    ::waitpid(pid,&status,0);
    char c;
    do
      {
      do
        {
        r=::read(output_p[0],&c,sizeof(char));
        } while((r==-1)&&(errno==EINTR));
      if(r&&(r!=-1)) result+=c;
      } while(r==sizeof(char));
    ::close(output_p[0]);
    if(result.len()&&(result[result.len()-1]=='\n'))
      result=result.subString(0,result.len()-1);
    }
  }
#endif
return(result);
}

Strings System::launchIE(const Strings & command,
                         const Strings & input) // static
{
Strings result;
#ifndef __MSDOS__
int input_p[2];
int error_p[2];
if(::pipe(input_p)==-1)
  {
  return(result);
  }
if(::pipe(error_p)==-1)
  {
  ::close(input_p[0]);
  ::close(input_p[1]);
  return(result);
  }
int r;
do
  {
  r=::write(input_p[1],(const char *)input,input.len());
  } while((r==-1)&&(errno==EINTR));
if(r==-1)
  {
  ::close(input_p[0]);
  ::close(input_p[1]);
  ::close(error_p[0]);
  ::close(error_p[1]);
  return(result);
  }
int pid=fork();
switch(pid)
  {
  case -1:
    {
    ::close(input_p[0]);
    ::close(input_p[1]);
    ::close(error_p[0]);
    ::close(error_p[1]);
    return(result);
    }
  case 0: // child
    {
    System::ignoreTimers();
    ::dup2(error_p[1],STDERR_FILENO);
    ::close(error_p[0]);
    ::close(error_p[1]);
    ::dup2(input_p[0],STDIN_FILENO);
    ::close(input_p[0]);
    ::close(input_p[1]);
    char * argv[4];
    argv[0]="/bin/sh";
    argv[1]="-c";
    argv[2]=(char *)(const char *)command;
    argv[3]=(char *)0;
    ::execv(argv[0],argv);
    ::exit(127); // should never be reached
    }
  default:
    {
    ::close(input_p[1]);
    ::close(error_p[1]);
    ::close(input_p[0]);
    int status;
    ::waitpid(pid,&status,0);
    char c;
    do
      {
      do
        {
        r=::read(error_p[0],&c,sizeof(char));
        } while((r==-1)&&(errno==EINTR));
      if(r&&(r!=-1)) result+=c;
      } while(r==sizeof(char));
    ::close(error_p[0]);
    if(result.len()&&(result[result.len()-1]=='\n'))
      result=result.subString(0,result.len()-1);
    }
  }
#endif
return(result);
}

Strings System::launchOE(const Strings & command,
                         Strings & err) // static
{
Strings result;
#ifndef __MSDOS__
int output_p[2];
int error_p[2];
err="";
if(::pipe(output_p)==-1)
  {
  return(result);
  }
if(::pipe(error_p)==-1)
  {
  ::close(output_p[0]);
  ::close(output_p[1]);
  return(result);
  }
int pid=fork();
switch(pid)
  {
  case -1:
    {
    ::close(output_p[0]);
    ::close(output_p[1]);
    ::close(error_p[0]);
    ::close(error_p[1]);
    return(result);
    }
  case 0: // child
    {
    System::ignoreTimers();
    ::dup2(output_p[1],STDOUT_FILENO);
    ::close(output_p[0]);
    ::close(output_p[1]);
    ::dup2(error_p[1],STDERR_FILENO);
    ::close(error_p[0]);
    ::close(error_p[1]);
    char * argv[4];
    argv[0]="/bin/sh";
    argv[1]="-c";
    argv[2]=(char *)(const char *)command;
    argv[3]=(char *)0;
    ::execv(argv[0],argv);
    ::exit(127); // should never be reached
    }
  default:
    {
    ::close(output_p[1]);
    ::close(error_p[1]);
    int status;
    ::waitpid(pid,&status,0);
    int r;
    char c;
    do
      {
      do
        {
        r=::read(output_p[0],&c,sizeof(char));
        } while((r==-1)&&(errno==EINTR));
      if(r&&(r!=-1)) result+=c;
      } while(r==sizeof(char));
    ::close(output_p[0]);
    do
      {
      do
        {
        r=::read(error_p[0],&c,sizeof(char));
        } while((r==-1)&&(errno==EINTR));
      if(r&&(r!=-1)) err+=c;
      } while(r==sizeof(char));
    ::close(error_p[0]);
    if(result.len()&&(result[result.len()-1]=='\n'))
      result=result.subString(0,result.len()-1);
    if(err.len()&&(err[err.len()-1]=='\n'))
      err=err.subString(0,err.len()-1);
    }
  }
#endif
return(result);
}

Strings System::launchIOE(const Strings & command,
                          const Strings & input,
                          Strings & err) // static
{
Strings result;
#ifndef __MSDOS__
int input_p[2];
int output_p[2];
int error_p[2];
err="";
if(::pipe(input_p)==-1)
  {
  return(result);
  }
if(::pipe(output_p)==-1)
  {
  ::close(input_p[0]);
  ::close(input_p[1]);
  return(result);
  }
if(::pipe(error_p)==-1)
  {
  ::close(input_p[0]);
  ::close(input_p[1]);
  ::close(output_p[0]);
  ::close(output_p[1]);
  return(result);
  }
int r;
do
  {
  r=::write(input_p[1],(const char *)input,input.len());
  } while((r==-1)&&(errno==EINTR));
if(r==-1)
  {
  ::close(input_p[0]);
  ::close(input_p[1]);
  ::close(output_p[0]);
  ::close(output_p[1]);
  ::close(error_p[0]);
  ::close(error_p[1]);
  return(result);
  }
int pid=fork();
switch(pid)
  {
  case -1:
    {
    ::close(input_p[0]);
    ::close(input_p[1]);
    ::close(output_p[0]);
    ::close(output_p[1]);
    ::close(error_p[0]);
    ::close(error_p[1]);
    return(result);
    }
  case 0: // child
    {
    System::ignoreTimers();
    ::dup2(error_p[1],STDERR_FILENO);
    ::close(error_p[0]);
    ::close(error_p[1]);
    ::dup2(output_p[1],STDOUT_FILENO);
    ::close(output_p[0]);
    ::close(output_p[1]);
    ::dup2(input_p[0],STDIN_FILENO);
    ::close(input_p[0]);
    ::close(input_p[1]);
    char * argv[4];
    argv[0]="/bin/sh";
    argv[1]="-c";
    argv[2]=(char *)(const char *)command;
    argv[3]=(char *)0;
    ::execv(argv[0],argv);
    ::exit(127); // should never be reached
    }
  default:
    {
    ::close(error_p[1]);
    ::close(output_p[1]);
    ::close(input_p[0]);
    ::close(input_p[1]);
    int status;
    ::waitpid(pid,&status,0);
    char c;
    do
      {
      do
        {
        r=::read(output_p[0],&c,sizeof(char));
        } while((r==-1)&&(errno==EINTR));
      if(r&&(r!=-1)) result+=c;
      } while(r==sizeof(char));
    ::close(output_p[0]);
    do
      {
      do
        {
        r=::read(error_p[0],&c,sizeof(char));
        } while((r==-1)&&(errno==EINTR));
      if(r&&(r!=-1)) err+=c;
      } while(r==sizeof(char));
    ::close(error_p[0]);
    if(result.len()&&(result[result.len()-1]=='\n'))
      result=result.subString(0,result.len()-1);
    if(err.len()&&(err[err.len()-1]=='\n'))
      err=err.subString(0,err.len()-1);
    }
  }
#endif
return(result);
}

