// @(#) ENIB/LI2 (c) - C++ Utilities
//--------------------------------------------------------------------------//
// file        : tcpUdp.cpp                                                 //
// description : source file for the ENIB/LI2 TCP and UDP communication     //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "Utility/tcpUdp.h"

#include <stdio.h>
#include <errno.h>

#ifndef __MSDOS__
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <netdb.h>
#include <net/if.h>
#include <unistd.h>
#include <signal.h>
#else
#define close(arg)            closesocket(arg)
#endif

#ifndef SHUT_RD
#define SHUT_RD 0
#endif
#ifndef SHUT_WR
#define SHUT_WR 1
#endif
#ifndef SHUT_RDWR
#define SHUT_RDWR 2
#endif

#ifndef __GNUG__
typedef int socklen_t;
#endif

//--------------------------------------------------------------------------//
//  class SocketElem (abstract)                                             //
//--------------------------------------------------------------------------//

SocketElem::IPMap SocketElem::_ipMap;

SimpleVector<int> SocketElem::_sockets;

bool SocketElem::_firstTime=true;
 
//-------- Static Methods --------------------------------------------------//
void SocketElem::relaxSockets(void) // static
{
SocketElem::_ipMap.erase(SocketElem::_ipMap.begin(),SocketElem::_ipMap.end());
for(int i=SocketElem::_sockets.getSize();i--;)
  {
  if(i>=0)
    {
    shutdown(SocketElem::_sockets[i],SHUT_RDWR);
    close(SocketElem::_sockets[i]);
    SocketElem::_sockets[i]=-1;
    }
  }
}
 
void * SocketElem::allocBytes(unsigned long nBytes) // static
{
return(BlockAllocator::allocate(nBytes));
}

void SocketElem::freeBytes(void * bytes) // static
{
BlockAllocator::free(bytes);
}

Strings SocketElem::getHostName(void) // static
{
Strings result;
char buffer[0x80];
if(gethostname(buffer,0x80)!=-1) result=buffer;
return(result);
}

Strings SocketElem::getIPAddress(const Strings & hostName) // static
{
Strings result;
unsigned long ipAddr=_getIPAddress(hostName);
if(ipAddr) _printAddress(result,ntohl(ipAddr));
return(result);
}

//-------- Allocators ------------------------------------------------------//
SocketElem::SocketElem(void) // protected
 : BlockAllocatedObject(),
   _socket(-1)
{
if(SocketElem::_firstTime)
  {
#ifndef __MSDOS__
  struct sigaction act;
  sigemptyset(&act.sa_mask);
  act.sa_handler=SIG_IGN;
  act.sa_flags=0;
  sigaction(SIGPIPE,&act,(struct sigaction *)0);
#else
  WSADATA wsaData;
  WORD wVersionRequested=MAKEWORD(1,1);
  int err=WSAStartup(wVersionRequested,&wsaData);
#endif
  SocketElem::_firstTime=false;
  }
}

SocketElem::~SocketElem(void) // protected
{
if(_socket!=-1) _closeSocket(_socket);
}

//-------- Inspectors ------------------------------------------------------//
bool SocketElem::fail(void) const
{
return(_socket==-1);
}
 
//-------- Modifiers -------------------------------------------------------//

//-------- Private Methods -------------------------------------------------//
unsigned long SocketElem::_getIPAddress(const Strings & hostName) // static
{
IPMap::const_iterator it=SocketElem::_ipMap.find(hostName);
if(it==SocketElem::_ipMap.end())
  {
  struct hostent * hp=gethostbyname(hostName);
  if(hp)
    {
    unsigned long ipAddr=*((unsigned long *)(hp->h_addr));
    SocketElem::_ipMap.insert(IPMap::value_type(hostName,ipAddr));
    return(ipAddr);
    }
  else return(0LU);
  }
else return((*it).second);
}

void SocketElem::_printAddress(Strings & result,unsigned long ipAddress)
{                                                                    // static
char buffer[0x10];
sprintf(buffer,"%lu.%lu.%lu.%lu",(ipAddress>>24)&0x000000FFUL,
                                 (ipAddress>>16)&0x000000FFUL,
                                 (ipAddress>>8)&0x000000FFUL,
                                 ipAddress&0x000000FFUL);
result=buffer;
}

void SocketElem::_closeSocket(int & socket) // static
{
for(int i=SocketElem::_sockets.getSize();i--;)
  {
  if(SocketElem::_sockets[i]==socket)
    {
    SocketElem::_sockets.erase(i);
    break;
    }
  }
shutdown(socket,SHUT_RDWR);
close(socket);
socket=-1;
}

//--------------------------------------------------------------------------//
//  class TCPListener                                                       //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
TCPListener::TCPListener(int portNumber)
 : SocketElem(),
   _acceptThread((SimpleThread *)0),
   _pendingMutex(), _pending()
{
_socket=socket(AF_INET,SOCK_STREAM,0);
if(_socket==-1)
  {
  return;
  }
struct sockaddr_in socketAddress;
socketAddress.sin_family=AF_INET;
socketAddress.sin_port=htons(portNumber);
socketAddress.sin_addr.s_addr=htonl(INADDR_ANY);
socklen_t addrLength=sizeof(struct sockaddr_in);
if(bind(_socket,(struct sockaddr *)&socketAddress,addrLength)==-1)
  {
  close(_socket);
  _socket=-1;
  return;
  }
if(getsockname(_socket,(struct sockaddr *)&socketAddress,&addrLength)==-1)
  {
  close(_socket);
  _socket=-1;
  return;
  }
_portNumber=ntohs(socketAddress.sin_port);
if(listen(_socket,10)==-1)
  {
  close(_socket);
  _socket=-1;
  return;
  }
char buffer[0x80];
if(gethostname(buffer,0x80)==-1)
  {
  close(_socket);
  _socket=-1;
  return;
  }
unsigned long ipAddr=_getIPAddress(buffer);
if(!ipAddr)
  {
  close(_socket);
  _socket=-1;
  return;
  }
_printAddress(_ipAddress,ntohl(ipAddr));
SocketElem::_sockets.pushBack(_socket);
_acceptThread=new SimpleThread(TCPListener::_acceptTask,this);
_acceptThread->start();
}

TCPListener::~TCPListener(void)
{
if(_acceptThread) delete _acceptThread;
while(_pending.size())
  {
  _closeSocket(_pending.back().first);
  _pending.pop_back();
  }
}

//-------- Inspectors ------------------------------------------------------//
int TCPListener::getNbPending(void) const
{
return(_pending.size());
}

int TCPListener::getPortNumber(void) const
{
return(_portNumber);
}

const Strings & TCPListener::getIPAddress(void) const
{
return(_ipAddress);
}

//-------- Modifiers -------------------------------------------------------//
TCPConnection * TCPListener::nextConnection(void)
{
TCPConnection * connection=(TCPConnection *)0;
_pendingMutex.lock();
if(_pending.size())
  {
  connection=new TCPConnection(_pending.front().first,   // socket
                               _pending.front().second); // address
  _pending.pop_front();
  }
_pendingMutex.unlock();
return(connection);
}

//-------- Private Methods -------------------------------------------------//
void * TCPListener::_acceptTask(void * data) // static
{
TCPListener * listener=(TCPListener *)data;
int sock;
struct sockaddr_in addr;
socklen_t addrLength=sizeof(struct sockaddr_in);
bool done=false;
while(!done)
  {
  sock=-1;
  do
    {
    sock=accept(listener->_socket,(sockaddr *)&addr,&addrLength);
    } while((sock==-1)&&(errno==EINTR));                              
  if(sock!=-1)
    {
    TCPConnection::_sockets.pushBack(sock);
    listener->_pendingMutex.lock();
    listener->_pending.push_back(PendingList::value_type(sock,addr));
    listener->_pendingMutex.unlock();
    // onConnection() CALLBACK
    }
  }
return((void *)0);
}

//--------------------------------------------------------------------------//
//  class TCPConnection                                                     //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
TCPConnection::TCPConnection(int portNumber,const Strings & host)
 : SocketElem(),
   _received(), _receiveMutex(),
   _receiveThread((SimpleThread *)0),
   _localPortNumber(-1), _distantPortNumber(-1),
   _localIPAddress(), _distantIPAddress()
{
_socket=socket(AF_INET,SOCK_STREAM,0);
if(_socket==-1)
  {
  return;
  }
unsigned long ipAddr=_getIPAddress(host);
if(!ipAddr)
  {
  close(_socket);
  _socket=-1;
  return;
  }
struct sockaddr_in socketAddress;
socketAddress.sin_family=AF_INET;
socketAddress.sin_port=htons(portNumber);
socketAddress.sin_addr.s_addr=ipAddr;
socklen_t addrLength=sizeof(struct sockaddr_in);
if(connect(_socket,(const sockaddr *)&socketAddress,addrLength)==-1)
  {
  close(_socket);
  _socket=-1;
  return;
  }
_distantPortNumber=ntohs(socketAddress.sin_port);
_printAddress(_distantIPAddress,ntohl(socketAddress.sin_addr.s_addr));
if(getsockname(_socket,(struct sockaddr *)&socketAddress,&addrLength)==-1)
  {
  close(_socket);
  _socket=-1;
  return;
  }
_localPortNumber=ntohs(socketAddress.sin_port);
_printAddress(_localIPAddress,ntohl(socketAddress.sin_addr.s_addr));
SocketElem::_sockets.pushBack(_socket);
_receiveThread=new SimpleThread(TCPConnection::_receiveTask,this);
_receiveThread->start();
}

TCPConnection::TCPConnection(int socket,                          // protected
                             struct sockaddr_in & socketAddress)
 : SocketElem(),
   _received(), _receiveMutex(),
   _receiveThread((SimpleThread *)0),
   _localPortNumber(-1), _distantPortNumber(-1),
   _localIPAddress(), _distantIPAddress()
{
_socket=socket;
_distantPortNumber=ntohs(socketAddress.sin_port);
_printAddress(_distantIPAddress,ntohl(socketAddress.sin_addr.s_addr));
socklen_t addrLength=sizeof(struct sockaddr_in);
if(getsockname(_socket,(struct sockaddr *)&socketAddress,&addrLength)==-1)
  {
  close(_socket);
  _socket=-1;
  return;
  }
_localPortNumber=ntohs(socketAddress.sin_port);
_printAddress(_localIPAddress,ntohl(socketAddress.sin_addr.s_addr));
_receiveThread=new SimpleThread(TCPConnection::_receiveTask,this);
_receiveThread->start();
}

TCPConnection::~TCPConnection(void)
{
if(_receiveThread) delete _receiveThread;
while(_received.size())
  {
  BlockAllocator::free(_received.back().second);
  _received.pop_back();
  }
}

//-------- Inspectors ------------------------------------------------------//
int TCPConnection::getLocalPortNumber(void) const
{
return(_localPortNumber);
}

const Strings & TCPConnection::getLocalIPAddress(void) const
{
return(_localIPAddress);
}

int TCPConnection::getDistantPortNumber(void) const
{
return(_distantPortNumber);
}

const Strings & TCPConnection::getDistantIPAddress(void) const
{
return(_distantIPAddress);
}

//-------- Modifiers -------------------------------------------------------//
bool TCPConnection::readBytes(void * & result,unsigned long & nBytes)
{
_receiveMutex.lock();
if(_received.size())
  {
  result=_received.front().second;
  nBytes=_received.front().first;
  _received.pop_front();
  _receiveMutex.unlock();
  return(true);
  }
else
  {
  _receiveMutex.unlock();
  return(false);
  }
}

bool TCPConnection::read(Strings & result)
{
_receiveMutex.lock();
if(_received.size())
  {
  char * bytes=(char *)_received.front().second;
  unsigned long nBytes=_received.front().first;
  _received.pop_front();
  if(nBytes)
    {
    bytes[nBytes-1]='\0'; // ensure string ends
    result=bytes;
    BlockAllocator::free(bytes);
    }
  else result="";
  _receiveMutex.unlock();
  return(true);
  }
else
  {
  _receiveMutex.unlock();
  return(false);
  }
}

void TCPConnection::writeBytes(const void * bytes,unsigned long nBytes)
{
int r;
if(_socket!=-1)
  {
  unsigned long length=htonl(nBytes);
  do
    {
    r=::send(_socket,(const char *)&length,sizeof(unsigned long),0);
    } while((r==-1)&&(errno==EINTR));
  if(r==-1) _closeSocket(_socket);
  }
if(_socket!=-1)
  {
  do
    {
    r=::send(_socket,(const char *)bytes,nBytes,0);
    } while((r==-1)&&(errno==EINTR));
  if(r==-1) _closeSocket(_socket);
  }
}

void TCPConnection::write(const Strings & str)
{
int r;
if(_socket!=-1)
  {
  unsigned long length=htonl(str.len()+1);
  do
    {
    r=::send(_socket,(const char *)&length,sizeof(unsigned long),0);
    } while((r==-1)&&(errno==EINTR));
  if(r==-1) _closeSocket(_socket);
  }
if(_socket!=-1)
  {
  do
    {
    r=::send(_socket,(const char *)str,str.len()+1,0);
    } while((r==-1)&&(errno==EINTR));
  if(r==-1) _closeSocket(_socket);
  }
}

//-------- Private Methods -------------------------------------------------//
void * TCPConnection::_receiveTask(void * data) // static
{
TCPConnection * connection=(TCPConnection *)data;
unsigned long nBytes;
bool done=false;
while(!done)
  {
  if(!connection->_readNBytes(&nBytes,sizeof(unsigned long)))
    {
    done=true;
    continue;
    }
  nBytes=ntohl(nBytes);
  char * buffer;
  if(nBytes)
    {
    buffer=(char *)BlockAllocator::allocate(nBytes);
    if(!connection->_readNBytes(buffer,nBytes)) done=true;
    }
  else buffer=(char *)0;
  connection->_receiveMutex.lock();
  connection->_received.push_back(BufferList::value_type(nBytes,buffer));
  connection->_receiveMutex.unlock();
  // onReception() CALLBACK
  }
connection->_closeSocket(connection->_socket);
return((void *)0);
}

bool TCPConnection::_readNBytes(void * buffer,unsigned long nBytes)
{
char * ptr=(char *)buffer;
int r;
while(nBytes)
  {
  r=::recv(_socket,ptr,nBytes,0);
  if(r>0)
    {
    nBytes-=r;
    ptr+=r;
    }
  else if((r==0)||(errno!=EINTR))
    return(false);
  }
return(true);
}

//--------------------------------------------------------------------------//
// class UDPTransmitter                                                     //
//--------------------------------------------------------------------------//
 
const int UDPTransmitter::_receptionSize=65535;

//-------- Allocators ------------------------------------------------------//
UDPTransmitter::UDPTransmitter(int portNumber)
 : SocketElem(),
   _received(), _receiveMutex(),
   _receiveThread((SimpleThread *)0),
   _receptionBuffer((char *)0),
   _portNumber(-1), _ipAddress()
{
_socket=socket(AF_INET,SOCK_DGRAM,0);
if(_socket==-1)
  {
  return;
  }
struct sockaddr_in localAddr;
localAddr.sin_family=AF_INET;
localAddr.sin_port=htons(portNumber);
localAddr.sin_addr.s_addr=htonl(INADDR_ANY);
socklen_t addrLength=sizeof(struct sockaddr_in);
#ifndef __MSDOS__
int on=1;
#else
char on=1;
#endif
if(setsockopt(_socket,SOL_SOCKET,SO_REUSEADDR,&on,sizeof(int))==-1)
  {
  close(_socket);
  _socket=-1;
  return;
  }
if(bind(_socket,(const sockaddr *)&localAddr,addrLength)==-1)
  {
  close(_socket);
  _socket=-1;
  return;
  }
_socketAddress=localAddr;
#ifndef __MSDOS__
_socketAddress.sin_addr.s_addr=ioctl(_socket,SIOCGIFBRDADDR,(void *)0);
#else
_socketAddress.sin_addr.s_addr=0xFFFFFFFFLU;
#endif
on=1;
if(setsockopt(_socket,SOL_SOCKET,SO_BROADCAST,&on,sizeof(int))==-1)
  {
  close(_socket);
  _socket=-1;
  return;
  }
struct sockaddr_in addr;
addrLength=sizeof(struct sockaddr_in);
if(getsockname(_socket,(struct sockaddr *)&addr,&addrLength)==-1)
  {
  close(_socket);
  _socket=-1;
  return;
  }
char buffer[0x80];
if(gethostname(buffer,0x80)==-1)
  {
  close(_socket);
  _socket=-1;
  return;
  }
unsigned long ipAddr=_getIPAddress(buffer);
if(!ipAddr)
  {
  close(_socket);
  _socket=-1;
  return;
  }
_portNumber=ntohs(addr.sin_port);
_printAddress(_ipAddress,ntohl(ipAddr));
SocketElem::_sockets.pushBack(_socket);
_receptionBuffer=(char *)BlockAllocator::allocate(_receptionSize);
_receiveThread=new SimpleThread(UDPTransmitter::_receiveTask,this);
_receiveThread->start();
}

UDPTransmitter::~UDPTransmitter(void)
{
if(_receiveThread) delete _receiveThread;
if(_receptionBuffer)
  BlockAllocator::free(_receptionBuffer);
while(_received.size())
  {
  BlockAllocator::free(_received.back().buffer);
  _received.pop_back();
  }
}
 
//-------- Inspectors ------------------------------------------------------//
int UDPTransmitter::getPortNumber(void) const
{
return(_portNumber);
}

const Strings & UDPTransmitter::getIPAddress(void) const
{
return(_ipAddress);
}
 
//-------- Modifiers -------------------------------------------------------//
bool UDPTransmitter::readBytes(void * & result,unsigned long & nBytes)
{
_receiveMutex.lock();
if(_received.size())
  {
  result=_received.front().buffer;
  nBytes=_received.front().nBytes;
  _received.pop_front();
  _receiveMutex.unlock();
  return(true);
  }
else
  {
  _receiveMutex.unlock();
  return(false);
  }
}

bool UDPTransmitter::readBytesFrom(void * & result,unsigned long & nBytes,
                                   int & portNumber,Strings & ipAddress)
{
_receiveMutex.lock();
if(_received.size())
  {
  result=_received.front().buffer;
  nBytes=_received.front().nBytes;
  _printAddress(ipAddress,_received.front().ipAddress);
  portNumber=_received.front().portNumber;
  _received.pop_front();
  _receiveMutex.unlock();
  return(true);
  }
else
  {
  _receiveMutex.unlock();
  return(false);
  }
}

bool UDPTransmitter::read(Strings & result)
{
_receiveMutex.lock();
if(_received.size())
  {
  char * bytes=(char *)_received.front().buffer;
  unsigned long nBytes=_received.front().nBytes;
  _received.pop_front();
  if(nBytes)
    {
    bytes[nBytes-1]='\0'; // ensure string ends
    result=bytes;
    BlockAllocator::free(bytes);
    }
  else result="";
  _receiveMutex.unlock();
  return(true);
  }
else
  {
  _receiveMutex.unlock();
  return(false);
  }
}

bool UDPTransmitter::readFrom(Strings & result,
                              int & portNumber,Strings & ipAddress)
{
_receiveMutex.lock();
if(_received.size())
  {
  _printAddress(ipAddress,_received.front().ipAddress);
  portNumber=_received.front().portNumber;
  char * bytes=(char *)_received.front().buffer;
  unsigned long nBytes=_received.front().nBytes;
  _received.pop_front();
  if(nBytes)
    {
    bytes[nBytes-1]='\0'; // ensure string ends
    result=bytes;
    BlockAllocator::free(bytes);
    }
  else result="";
  _receiveMutex.unlock();
  return(true);
  }
else
  {
  _receiveMutex.unlock();
  return(false);
  }
}

void UDPTransmitter::writeBytesTo(const void * bytes,unsigned long nBytes,
                                  int portNumber,const Strings & host)
{
int r;
if(_socket!=-1)
  {
  unsigned long ipAddr=_getIPAddress(host);
  if(ipAddr)
    {
    struct sockaddr_in destAddress;
    destAddress.sin_family=AF_INET;
    destAddress.sin_port=htons(portNumber);
    destAddress.sin_addr.s_addr=ipAddr;
    do
      {
      r=::sendto(_socket,(const char *)bytes,nBytes,0,
                 (struct sockaddr *)&destAddress,sizeof(struct sockaddr_in));
      } while((r==-1)&&(errno==EINTR));
    if(r==-1) _closeSocket(_socket);
    }
  }
}

void UDPTransmitter::writeTo(const Strings & str,
                             int portNumber,const Strings & host)
{
int r;
if(_socket!=-1)
  {
  unsigned long ipAddr=_getIPAddress(host);
  if(ipAddr)
    {
    struct sockaddr_in destAddress;
    destAddress.sin_family=AF_INET;
    destAddress.sin_port=htons(portNumber);
    destAddress.sin_addr.s_addr=ipAddr;
    do
      {
      r=::sendto(_socket,(const char *)str,str.len()+1,0,
                 (struct sockaddr *)&destAddress,sizeof(struct sockaddr_in));
      } while((r==-1)&&(errno==EINTR));
    if(r==-1) _closeSocket(_socket);
    }
  }
}

void UDPTransmitter::broadcastBytes(const void * bytes,unsigned long nBytes)
{
int r;
if(_socket!=-1)
  {
  do
    {
    r=::sendto(_socket,(const char *)bytes,nBytes,0,
               (struct sockaddr *)&_socketAddress,sizeof(struct sockaddr_in));
    } while((r==-1)&&(errno==EINTR));
  if(r==-1) _closeSocket(_socket);
  }
}
 
void UDPTransmitter::broadcast(const Strings & str)
{
int r;
if(_socket!=-1)
  {
  do
    {
    r=::sendto(_socket,(const char *)str,str.len()+1,0,
               (struct sockaddr *)&_socketAddress,sizeof(struct sockaddr_in));
    } while((r==-1)&&(errno==EINTR));
  if(r==-1) _closeSocket(_socket);
  }
}

//-------- Private Methods -------------------------------------------------//
void * UDPTransmitter::_receiveTask(void * data) // static
{
UDPTransmitter * broadcaster=(UDPTransmitter *)data;
int nBytes;
bool done=false;
while(!done)
  {
  struct sockaddr_in addr;
  socklen_t addrLength=sizeof(struct sockaddr_in);
  do
    {
    nBytes=recvfrom(broadcaster->_socket,broadcaster->_receptionBuffer,
                    _receptionSize,0,
                    (struct sockaddr *)&addr,(socklen_t *)&addrLength);
    } while((nBytes==-1)&&(errno==EINTR));
  if(nBytes==-1)
    {
    done=true;
    continue;
    }
  Buffer b;
  b.nBytes=nBytes;
  if(nBytes)
    {
    b.buffer=(char *)BlockAllocator::allocate(nBytes);
    memcpy(b.buffer,broadcaster->_receptionBuffer,nBytes);
    }
  else b.buffer=(char *)0;
  b.ipAddress=ntohl(addr.sin_addr.s_addr);
  b.portNumber=ntohs(addr.sin_port);
  broadcaster->_receiveMutex.lock();
  broadcaster->_received.push_back(b);
  broadcaster->_receiveMutex.unlock();
  // onReception() CALLBACK
  }
broadcaster->_closeSocket(broadcaster->_socket);
return((void *)0);
}

