// @(#) ENIB/LI2 (c) - C++ Utilities
//--------------------------------------------------------------------------//
// file        : timer.cpp                                                  //
// description : source file for the ENIB/LI2 class Timer                   //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "Utility/timer.h"

#ifndef __MSDOS__
#include <signal.h>
#endif

//--------------------------------------------------------------------------//
// class Timer                                                              //
//--------------------------------------------------------------------------//

int Timer::_nbTimers=0;

unsigned long Timer::_currentId=0;

#ifndef __MSDOS__
struct sigaction Timer::_newAction;
struct sigaction Timer::_oldAction;
#endif

SimpleVector<Timer::Job *> Timer::_jobStock;

StlList<Timer::Job *> Timer::_jobs;

//-------- Allocators ------------------------------------------------------//

Timer::Timer(void)
 : BlockAllocatedObject()
{
if(!_nbTimers++)
  {
#ifndef __MSDOS__
  Timer::_newAction.sa_handler=Timer::_timerHandler;
  sigemptyset(&Timer::_newAction.sa_mask);
  Timer::_newAction.sa_flags=0;
  ::sigaction(SIGALRM,&Timer::_newAction,&Timer::_oldAction);
#endif
  }
}

Timer::~Timer(void)
{
if(!--_nbTimers)
  {
  Timer::_stop();
#ifndef __MSDOS__
  ::sigaction(SIGALRM,&Timer::_oldAction,(struct sigaction *)0);
#endif
  while(Timer::_jobs.size())
    {
    delete Timer::_jobs.front();
    Timer::_jobs.pop_front();
    }
  while(Timer::_jobStock.getSize())
    {
    delete Timer::_jobStock.readPopBack();
    }
  }
}

//-------- Time management -------------------------------------------------//
// inline void Timer::sleep(unsigned long ms) // static

// inline unsigned long Timer::getTime(void) // static

unsigned long Timer::addTimeOut(unsigned long ms,
                                Timer::Callback timerCB,void * timerData)
{
if(Timer::_jobs.size()) Timer::_stop();
Timer::Job * job=(Timer::_jobStock.getSize() ? Timer::_jobStock.readPopBack()
                                             : new Timer::Job);
if(!(++Timer::_currentId)) // avoid 0 when _currentId loops
  Timer::_currentId++;
job->id=Timer::_currentId;
unsigned long date=Timer::getTime();
job->date=date+ms;
job->cb=timerCB;
job->data=timerData;
StlList<Timer::Job *>::iterator it=Timer::_jobs.begin();
for(;it!=Timer::_jobs.end();it++)
  {
  if((*it)->date>=job->date) break; // insert here
  }
Timer::_jobs.insert(it,job);
Timer::_relaunch(date);
return(Timer::_currentId);
}

bool Timer::removeTimeOut(unsigned long id)
{
bool found=false;
if(Timer::_jobs.size()) Timer::_stop();
StlList<Timer::Job *>::iterator it=Timer::_jobs.begin();
for(;it!=Timer::_jobs.end();it++)
  {
  if((*it)->id==id)
    {
    Timer::_jobStock.pushBack(*it);
    Timer::_jobs.remove(*it);
    found=true;
    break;
    }
  }
Timer::_relaunch(Timer::getTime());
return(found);
}

//-------- Private methods -------------------------------------------------//

void Timer::_timerHandler(int sigNum) // static
{
unsigned long date=Timer::getTime();
#ifndef __MSDOS__
struct itimerval itv;
#endif
Timer::Job * job;
while(Timer::_jobs.size())
  {
  job=Timer::_jobs.front();
  if(job->date<=date)
    {
    Timer::_jobs.pop_front();
    (*(job->cb))(job->data);
    Timer::_jobStock.pushBack(job);
    }
  else
    {
    date=job->date-date;
#ifndef __MSDOS__
    itv.it_value.tv_sec=date/1000LU;
    itv.it_value.tv_usec=1000LU*(date%1000LU);
    itv.it_interval.tv_sec=itv.it_interval.tv_usec=0LU;
    setitimer(ITIMER_REAL,&itv,(struct itimerval *)0);
#endif
    break;
    }
  }
#ifndef __MSDOS__
if(Timer::_oldAction.sa_handler) (*Timer::_oldAction.sa_handler)(sigNum);
#endif
}

void Timer::_stop(void) // static
{
#ifndef __MSDOS__
struct itimerval itv={{0LU,0LU},{0LU,0LU}};
setitimer(ITIMER_REAL,&itv,(struct itimerval *)0);
#endif
}

void Timer::_relaunch(unsigned long date) // static
{
bool tooLate=false;
unsigned long ms=0LU;
StlList<Timer::Job *>::iterator it=Timer::_jobs.begin();
for(;it!=Timer::_jobs.end();it++)
  {
  if((*it)->date>date)
    {
    ms=(*it)->date-date;
    break;
    }
  else tooLate=true;
  }
if(tooLate)
  {
#ifndef __MSDOS__
  Timer::_timerHandler(SIGALRM);
#endif
  }
if(ms)
  {
#ifndef __MSDOS__
  struct itimerval itv;
  itv.it_value.tv_sec=ms/1000LU;
  itv.it_value.tv_usec=1000LU*(ms%1000LU);
  itv.it_interval.tv_sec=itv.it_interval.tv_usec=0LU;
  setitimer(ITIMER_REAL,&itv,(struct itimerval *)0);
#endif
  }
}

