// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : xsPoint.cpp                                                //
// description : source file for the ENIB/LI2 class XGuiPoint               //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#ifdef X_GUI

#include "XGui/xsPoint.h"
#include "XGui/xgView2d.h"
#include "XGui/xgMgr.h"

//--------------------------------------------------------------------------//
// class XGuiPoint                                                          //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
XGuiPoint::XGuiPoint(double x,double y,double theta,const Strings & color)
 : XGuiShape2D(x,y,theta,color)
{
computePosition();
}

XGuiPoint::XGuiPoint(const XGuiPoint & s)
 : XGuiShape2D(s)
{}

XGuiPoint::~XGuiPoint(void)
{}

XGuiShape2D * XGuiPoint::duplicate(void) const // virtual
{
return(new XGuiPoint(*this));
}

//-------- Inspectors ------------------------------------------------------//
// inline void XGuiPoint::getLocalBoundingBox(double & xmin,double & ymin,
//                                            double & xmax,double & ymax)
//                                            const // virtual

// inline void XGuiPoint::getBoundingBox(double & xmin,double & ymin,
//                                       double & xmax,double & ymax,
//                                       const XGuiViewer2D * viewer) const
//                                                               // virtual

// inline double XGuiPoint::distanceTo(double x,double y,
//                                     const XGuiViewer2D * viewer)
//                                                        const // virtual

bool XGuiPoint::isInside(double x,double y) const // virtual
{
return((x==_x0)&&(y==_y0));
}

bool XGuiPoint::intersectRay(double xRay,double yRay,double xDir,double yDir,
                             double & xInter,double & yInter) const // virtual
{
double a=0.0,b=0.0,c=0.0;
XGuiShape2D::_equationLine(xRay,yRay,xRay+xDir,yRay+yDir,a,b,c);
if(a*_x0+b*_y0+c) return(false); // not on the line
double dx=_x0-xRay;
double dy=_y0-yRay;
if(dx*xDir+dy*yDir<=0.0) return(false); // bad direction
xInter=_x0;
yInter=_y0;
return(true);
}

//-------- Input/Output ----------------------------------------------------//
Strings XGuiPoint::print(void) const // virtual
{
return("point()");
}

//-------- Modifiers -------------------------------------------------------//
void XGuiPoint::drawInViewer(XGuiViewer2D * viewer) // virtual
{
getXGuiManager()->chooseColor(_pixel);
XDrawPoint(getXGuiManager()->getDisplay(),
           viewer->getPixmap(),getXGuiManager()->getGC(),
           viewer->getColumn(_x0),
           viewer->getLine(_y0));
}

// inline void XGuiPoint::computePosition(void) // virtual

#endif // X_GUI

